@@ -2,8 +2,8 @@
 #'
 #' Fast similarity/distance computation function for large sparse matrices. You
 #' can floor small similarity value to to save computation time and storage
-#' space by an arbitrary threashold (\code{min_simil}) or rank (\code{rank}).
-#' Please increase the numbner of threads for better perfromance using
+#' space by an arbitrary threshold (\code{min_simil}) or rank (\code{rank}).
+#' Please increase the number of threads for better performance using
 #' \code{\link[RcppParallel]{setThreadOptions}}.
 #'
 #' @param x \link{Matrix} object
@@ -12,12 +12,15 @@
 #' @param margin integer indicating margin of similarity/distance computation. 1
 #'   indicates rows or 2 indicates columns.
 #' @param method method to compute similarity or distance
-#' @param min_simil the minimum similarity value to be recoded.
+#' @param min_simil the minimum similarity value to be recorded.
 #' @param rank an integer value specifying top-n most similarity values to be
 #'   recorded.
-#' @param p weight for minkowski distance
+#' @param p weight for Minkowski distance
 #' @param drop0 if \code{TRUE}, zero values are removed regardless of
 #'   \code{min_simil} or \code{rank}.
+#' @param diag if \code{TRUE}, only compute diagonal elements of the
+#' similarity/distance matrix; useful when comparing corresponding rows or columns
+#' of `x` and `y`.
 #' @param digits determines rounding of small values towards zero. Use primarily
 #'   to correct rounding errors in C++. See \link{zapsmall}.
 #' @import methods Matrix
@@ -30,25 +33,29 @@
 simil <- function(x, y = NULL, margin = 1,
                   method = c("cosine", "correlation", "jaccard", "ejaccard",
                              "dice", "edice", "hamman", "simple matching", "faith"),
-                  min_simil = NULL, rank = NULL, drop0 = FALSE, digits = 14) {
+                  min_simil = NULL, rank = NULL, drop0 = FALSE, diag = FALSE, digits = 14) {
 
     method <- match.arg(method)
-    proxy(x, y, margin, method, min_proxy = min_simil, rank = rank, drop0 = drop0, digits = digits)
+    proxy(x, y, margin, method, min_proxy = min_simil, rank = rank, drop0 = drop0,
+          diag = diag, digits = digits)
 
 }
 
 #' @rdname simil
+#' @param smooth adds a  fixed value to all the cells to avoid division by zero.
+#'   Only used when `method` is "chisquared" or "kullback".
 #' @export
 #' @examples
 #' mt <- Matrix::rsparsematrix(100, 100, 0.01)
 #' dist(mt, method = "euclidean")[1:5, 1:5]
 dist <- function(x, y = NULL, margin = 1,
-                 method = c("euclidean", "chisquared", "hamming", "kullback",
-                            "manhattan", "maximum", "canberra", "minkowski"),
-                 p = 2, drop0 = FALSE, digits = 14) {
+                 method = c("euclidean", "chisquared", "kullback",
+                            "manhattan", "maximum", "canberra", "minkowski", "hamming"),
+                 p = 2, smooth = 0, drop0 = FALSE, diag = FALSE, digits = 14) {
 
     method <- match.arg(method)
-    proxy(x, y, margin, method, p = p, drop0 = drop0, digits = digits)
+    proxy(x, y, margin, method, p = p, smooth = smooth, drop0 = drop0,
+          diag = diag, digits = digits)
 }
 
 #' @import Rcpp
@@ -56,9 +63,9 @@ dist <- function(x, y = NULL, margin = 1,
 proxy <- function(x, y = NULL, margin = 1,
                   method = c("cosine", "correlation", "jaccard", "ejaccard",
                              "dice", "edice", "hamman", "simple matching", "faith",
-                             "euclidean", "chisquared", "hamming", "kullback",
-                             "manhattan", "maximum", "canberra", "minkowski"),
-                  p = 2, min_proxy = NULL, rank = NULL, drop0 = FALSE, digits = 14) {
+                             "euclidean", "chisquared", "kullback",
+                             "manhattan", "maximum", "canberra", "minkowski", "hamming"),
+                  p = 2, smooth = 0, min_proxy = NULL, rank = NULL, drop0 = FALSE, diag = FALSE, digits = 14) {
 
     method <- match.arg(method)
     if(is(x, 'sparseMatrix')) {
@@ -123,7 +130,7 @@ proxy <- function(x, y = NULL, margin = 1,
         x <- as(as(x, "lgCMatrix"), "dgCMatrix")
         y <- as(as(y, "lgCMatrix"), "dgCMatrix")
     }
-    if (method %in% c("cosine", "correlation", "euclidean")) {
+    if (method %in% c("cosine", "correlation", "euclidean") && !diag) {
         result <- cpp_linear(
             mt1 = x,
             mt2 = y,
@@ -137,25 +144,30 @@ proxy <- function(x, y = NULL, margin = 1,
         result <- cpp_pair(
             mt1 = x,
             mt2 = y,
-            method = match(method, c("ejaccard", "edice", "hamman", "simple matching",
-                                     "faith", "chisquared", "kullback", "manhattan",
-                                     "maximum", "canberra", "minkowski")),
+            method = match(method, c("cosine", "correlation", "ejaccard", "edice",
+                                     "hamman", "simple matching", "faith",
+                                     "euclidean", "chisquared", "kullback", "manhattan",
+                                     "maximum", "canberra", "minkowski", "hamming")),
             rank = rank,
             limit = min_proxy,
             weight = weight,
+            smooth = smooth,
             symm = symm,
+            diag = diag,
             drop0 = drop0
         )
     }
+    if (diag)
+        result <- as(as(result, "diagonalMatrix"), "ddiMatrix")
     result@x <- zapsmall(result@x, digits)
     dimnames(result) <- list(colnames(x), colnames(y))
     return(result)
 }
 
-#' Standard deviasion of columns and rows in sparse matrix
+#' Standard deviation of columns and rows in sparse matrix
 #'
 #' Produces the same result as \code{apply(x, 1, sd)} or \code{apply(x, 2, sd)}
-#' as without coecirng matrix to dense matrix. Values are not identical to
+#' as without coercing matrix to dense matrix. Values are not identical to
 #' \code{sd} because of the floating point precision issue in C++.
 #' @param x \link{Matrix} object
 #' @examples
