@@ -1,6 +1,5 @@
-context("test simil")
-
-mat_test <- Matrix::rsparsematrix(100, 100, 0.5)
+require(Matrix)
+mat_test <- rsparsematrix(100, 90, 0.5)
 
 test_simil <- function(x, method, margin, ignore_upper = FALSE, ignore_diag = TRUE, ...) {
     # test with only x
@@ -47,55 +46,55 @@ test_simil <- function(x, method, margin, ignore_upper = FALSE, ignore_diag = TR
     expect_equal(as.numeric(s3), as.numeric(s4), tolerance = 0.001)
 }
 
-test_that("test textstat_proxy cosine similarity", {
+test_that("test cosine similarity", {
     skip_if_not_installed("proxy")
     test_simil(mat_test, "cosine", margin = 1)
     test_simil(mat_test, "cosine", margin = 2)
 })
 
-test_that("test textstat_proxy correlation similarity", {
+test_that("test correlation similarity", {
     skip_if_not_installed("proxy")
     test_simil(mat_test, "correlation", margin = 1)
     test_simil(mat_test, "correlation", margin = 2)
 })
 
-test_that("test textstat_proxy jaccard similarity", {
+test_that("test jaccard similarity", {
     skip_if_not_installed("proxy")
     test_simil(mat_test, "jaccard", margin = 1)
     test_simil(mat_test, "jaccard", margin = 2)
 })
 
-test_that("test textstat_proxy ejaccard similarity", {
+test_that("test ejaccard similarity", {
     skip_if_not_installed("proxy")
     test_simil(mat_test, "ejaccard", margin = 1)
     test_simil(mat_test, "ejaccard", margin = 2)
 })
 
-test_that("test textstat_proxy dice similarity", {
+test_that("test dice similarity", {
     skip_if_not_installed("proxy")
     test_simil(mat_test, "dice", margin = 1)
     test_simil(mat_test, "dice", margin = 2)
 })
 
-test_that("test textstat_proxy edice similarity", {
+test_that("test edice similarity", {
     skip_if_not_installed("proxy")
     test_simil(mat_test, "edice", margin = 1)
     test_simil(mat_test, "edice", margin = 2)
 })
 
-test_that("test textstat_proxy simple matching similarity", {
+test_that("test simple matching similarity", {
     skip_if_not_installed("proxy")
     test_simil(mat_test, "simple matching", margin = 1)
     test_simil(mat_test, "simple matching", margin = 2)
 })
 
-test_that("test textstat_proxy hamman similarity", {
+test_that("test hamman similarity", {
     skip_if_not_installed("proxy")
     test_simil(mat_test, "hamman", margin = 1)
     test_simil(mat_test, "hamman", margin = 2)
 })
 
-test_that("test textstat_proxy faith similarity", {
+test_that("test faith similarity", {
     skip_if_not_installed("proxy")
     test_simil(mat_test, "faith", margin = 1, ignore_upper = TRUE)
     test_simil(mat_test, "faith", margin = 2, ignore_upper = TRUE)
